__author__ = "branko@toic.org (http://toic.org)"
__date__ = "Dec 9, 2012 2:00 PM$"
__version__ = "0.2.1b"

import ConfigParser
import os
import sys
from operator import itemgetter


"""
Now lets import some dependencies and check their versions
We need at least version 3.0.7a of BeautifulSoup
"""
try:
    import lxml.html #@UnresolvedImport
except:
    print "No lxml package found..."
    print "please install lxml >= 3.x.x"
    print "best way to install easy_install lxml"
    sys.exit(1)

class ApacheStatus(object):
    def __init__(self):
        """
        Try to detect config file location, trying to use user defined config
        in home path ~/.aptop.conf or /etc/aptop.conf then
        """
        homedir = os.path.expanduser('~')

        if os.path.isfile(os.path.join(homedir, '.aptop.conf')):
            self.configfile = os.path.join(homedir, '.aptop.conf')
        else:
            self.configfile = None

        """ 
        Let's populate some defaults if no config file is found
        """
        if not self.configfile:
            if os.path.isdir('/var/cpanel'):
                self.status_url = 'http://localhost/whm-server-status'
            else:
                self.status_url = 'http://localhost/server-status'
            self.refresh = '5'
        else:
            config = ConfigParser.ConfigParser()
            config.readfp(open(self.configfile))
            try:
                self.status_url = config.get('aptop', 'status_url')
                self.refresh = config.get('aptop', 'refresh')
            except:
                if os.path.isdir('/var/cpanel'):
                    self.status_url = 'http://localhost/whm-server-status'
                else:
                    self.status_url = 'http://localhost/server-status'
                self.refresh = '5'
        try:
            self.tree = lxml.html.parse(self.status_url)
        except:
            print "Apache not running or wrong mod_status url!?"
            sys.exit(1)

        # define should we filter out inactive sessions
        self.active = True
        self.sort_by = 'SS'
        # this is passed to reverse parameter on sort(list)
        self.sort_order = False

    def reverse_order(self):
        """ just reverese the ordering of current sorting """

        if self.sort_order:
            self.sort_order = False
        else:
            self.sort_order = True

    def refresh_rate(self):
        """
        (NoneType) -> int
        
        Returns parsed refresh time interval as int
        """
        return int(self.refresh)

    def fetch_status(self):
        """
        (NoneType) -> NoneType
        
        Refetching of data
        """
        if self.tree:
            old_tree = self.tree
        try:

            self.tree = lxml.html.parse(self.status_url)
        except:
            self.tree = old_tree

    def verify_mod_status(self):
        """
        (str) -> boolean
        
        Checks the data string for Apache Status title and returns true or
        false
        """
        return self.tree.find('.//title').text == 'Apache Status'

    def count_by_vhost(self, data):
        """
        (str) -> list of tuple
        
        Counts the active concurent connections by vhosts and returns an ordered
        list of tuples containing vhost name and number of active connections
        """
        vstatus = {}
        if self.active:
            vhosts = self.filter_active(data)
        else:
            vhosts = data
        for status in vhosts:
            if status['VHost'] in vstatus:
                vstatus[status['VHost']] += 1
            else:
                vstatus[status['VHost']] = 1
        items = vstatus.items()
        items.sort(key=itemgetter(1), reverse=True)

        return items

    def count_by_client(self, data):
        """
        (str) -> list of tuple
        
        Counts the active concurent connections by clients and returns an ordered
        list of tuples containing client IP and number of active connections
        """
        cstatus = {}
        if self.active:
            vhosts = self.filter_active(data)
        else:
            vhosts = data
        for status in vhosts:
            if status['Client'] in cstatus:
                cstatus[status['Client']] += 1
            else:
                cstatus[status['Client']] = 1
        items = cstatus.items()
        items.sort(key=itemgetter(1), reverse=True)

        return items

    def togle_active(self):
        if self.active:
            self.active = False
        else:
            self.active = True


    def filter_active(self, data):
        """
        (list of dict) -> list of dict
        
        Returns list of dicts based on filter status
        
        """
        filtered = []

        for status in data:
            if status['M'] != '_' and status['M'] != '.':
                filtered.append(status)
        return filtered


    def display_vhosts(self, data):
        """
        (list of dict) -> list of dict
        retrurn vhost data as list of dicts based on current filter 
        """
        if self.active:
            results = self.filter_active(data)
        else:
            results = data
        return results


    def parse_vhosts(self):
        """
        (str) -> list of dict
        
        Parses a apache status from internal class self.tree variable
        and returns list ofdict containing all vhost informations from 
        mod_status
        
        """
        tree = self.tree.xpath('//table[@border="0"]')[0]
        vhost_status = []
        headers = tree.findall('.//th')
        h2 = [s.text_content().replace('\n', '') for s in headers]
        for row in tree.findall('.//tr')[1:]: # this is header, excluding
            d = [s.text_content().replace('\n', '') for s in row.findall('.//td')]
            vhost_status.append(dict(zip(h2, d)))

        return sorted(vhost_status,
                      key=lambda k: float(k[self.sort_by]),
                      reverse=self.sort_order)

    def parse_header(self):
        """
        (NoneType) -> list
        
        Returns a list of header status variables from apache status page
        """

        headers = [h.text.replace('\n', '') for h in self.tree.findall('.//dt')]
        return headers



__author__ = "branko@toic.org (http://toic.org)"
__date__ = "Dec 9, 2012 2:00 PM$"
__version__ = "0.2.1b"

import curses
import time
import sys

HEADER_HEIGHT = 10
FOOTER_HEIGHT = 2


class AptopCurses(object):
	def __init__(self, aptop):
		self.stdscr = curses.initscr()
		self.aptop = aptop
		self.MAX_H, self.MAX_W = self.stdscr.getmaxyx()

		self.stdscr.nodelay(1)
		self.refresh = self.aptop.refresh_rate()
		self.counter = 0
		self.BODY_H = self.MAX_H
		self.view = 'H'
		curses.wrapper(self.start)


	def start(self, stdscr):
		if self.aptop.verify_mod_status():
			while 1:
				self.aptop.fetch_status()
				self.BODY_H = self.MAX_H - HEADER_HEIGHT - FOOTER_HEIGHT
				c = self.stdscr.getch()
				if c == ord('q') or c == ord('Q'): break
				elif c == ord('v') or c == ord('V'):
					self.view = 'V'
				elif c == ord('h') or c == ord('H'):
					self.view = 'H'
				elif c == ord('c') or c == ord('C'):
					self.view = 'C'
				elif c == ord('i') or c == ord('I'):
					self.aptop.togle_active()
				elif c == ord('r') or c == ord('R'):
					self.aptop.reverse_order()
				self.draw_view()
		else:
			print "Apache not running or wrong mod_status url!"
			sys.exit(1)

	def draw_view(self):
		self.draw_header()

		if self.view == 'V':
			self.draw_vhosts()
		elif self.view == 'H':
			self.draw_dashboard()
		elif self.view == 'C':
			self.draw_clients()
		else:
			print "something went wrong"
			sys.exit(1)
		self.draw_footer()
		self.iterate()

	def iterate(self):
		curses.napms(self.refresh * 1000)
		#time.sleep(self.refresh)
		self.MAX_H, self.MAX_W = self.stdscr.getmaxyx()



	def draw_header(self):
		""" draws a header window """
		header = curses.newwin(
								HEADER_HEIGHT,
								self.MAX_W,
								0, 0
								)
		header_data = self.aptop.parse_header()
		hcount = 0
		for header_line in header_data:
			try:
				header.addstr(hcount, 1, str(header_line[:self.MAX_W - 2]))
			except curses.error:
				pass
			hcount += 1
		header.refresh()

	def draw_dashboard(self):
		""" draws a dashboard window """
		dash = curses.newwin(
							self.BODY_H,
							self.MAX_W,
							HEADER_HEIGHT, 0
							)

		dash_data = self.aptop.display_vhosts(self.aptop.parse_vhosts())
		dcount = 0
		try:
			formatstr = '%-5s %1s %7s %3s %6s %7s %12s %-15s %-25s %-50s' % (
				'PID', 'M', 'CPU', 'SS', 'Req', 'Conn', 'Acc', 'Client', 'VHost',
				'Request' + ' ' * self.MAX_W)

			dash.addstr(0, 0, formatstr, curses.A_REVERSE)
		except curses.error:
			pass
		for dash_line in dash_data:
			dcount += 1
			try:
				formatstr = '%-5s %1s %7s %3s %6s %7s %12s %-15s %-25s %s' % (
				dash_line['PID'], dash_line['M'], dash_line['CPU'],
				dash_line['SS'], dash_line['Req'], dash_line['Conn'],
				dash_line['Acc'], dash_line['Client'], dash_line['VHost'][:26],
				dash_line['Request'])
				dash.addstr(dcount, 0, formatstr)
			except curses.error:
				pass
		dash.refresh()

	def draw_vhosts(self):
		""" draws a vhosts window """
		body = curses.newwin(
							self.BODY_H,
							self.MAX_W,
							HEADER_HEIGHT, 0
							)

		body_data = self.aptop.count_by_vhost(self.aptop.parse_vhosts())
		bcount = 0
		try:
			body.addstr(0, 0, str(" ") * self.MAX_W, curses.A_REVERSE)
			body.addstr(bcount, 1, 'Num req', curses.A_REVERSE)
			body.addstr(bcount, 20, "Virtualhost", curses.A_REVERSE)
		except curses.error:
			pass
		for body_line in body_data:
			bcount += 1
			try:
				body.addstr(bcount, 1, str(body_line[1]))
				body.addstr(bcount, 20, str(body_line[0]))
			except curses.error:
				pass
		body.refresh()

	def draw_clients(self):
		""" draws a vhosts window """
		body = curses.newwin(
							self.BODY_H,
							self.MAX_W,
							HEADER_HEIGHT, 0
							)

		body_data = self.aptop.count_by_client(self.aptop.parse_vhosts())
		bcount = 0
		try:
			body.addstr(0, 0, str(" ") * self.MAX_W, curses.A_REVERSE)
			body.addstr(bcount, 1, 'Num req', curses.A_REVERSE)
			body.addstr(bcount, 20, "Client IP", curses.A_REVERSE)
		except curses.error:
			pass
		for body_line in body_data:
			bcount += 1
			try:
				body.addstr(bcount, 1, str(body_line[1]))
				body.addstr(bcount, 20, str(body_line[0]))
			except curses.error:
				pass
		body.refresh()

	def draw_footer(self):
		""" draws a footer window """
		footer = curses.newwin(
								FOOTER_HEIGHT,
								self.MAX_W,
								self.MAX_H - FOOTER_HEIGHT, 0
								)
		try:
			footer.addstr(1, 1, 'Q')
			footer.addstr(1, 3, 'Quit ApTOP', curses.A_REVERSE)
			footer.addstr(1, 14, 'H')
			footer.addstr(1, 16, 'Home', curses.A_REVERSE)
			footer.addstr(1, 26, 'V')
			footer.addstr(1, 28, 'By Vhost', curses.A_REVERSE)
			footer.addstr(1, 37, 'C')
			footer.addstr(1, 39, 'By Client', curses.A_REVERSE)
		except curses.error:
			pass
		footer.refresh()

